#ifndef PERF01_API_H
#define PERF01_API_H

/* RDM 11.0.0.1000 */

#include "Db.h"
#include "Cursor.h"
#include "perf01.h"

namespace RDM_CPP {

class Cursor_simple;

/*! \brief The perf01 Db class
 *
 * This class provides the implementation for the perf01 database
*/
class Db_perf01 : public Db {
public:
    /*! \brief Open the perf01 database using the default name
     *
     * \param mode The open mode to use
     * \return A validated Db_perf01 object
     */
    static Db_perf01 Open (char *mode="s") throw (rdm_exception&);

    /*! \brief Open the perf01 database using the default name
     *
     * \param mode The open mode to use
     * \return A validated Db_perf01 object
     */
#if !defined(RDM_NO_UNICODE)
    static Db_perf01 Open (wchar_t *mode=L"s") throw (rdm_exception&);
#endif /*RDM_NO_UNICODE*/

    /*! \brief Open the perf01 database using the specified name
     *
     * \param name The database name to use for the open call
     * \param mode The open mode to use
     * \return A validated Db_perf01 object
     */
    static Db_perf01 Open (const char *name, const char *mode="s") throw (rdm_exception&);

    /*! \brief Open the perf01 database using the specified name
     *
     * \param name The database name to use for the open call
     * \param mode The open mode to use
     * \return A validated Db_perf01 object
     */
#if !defined(RDM_NO_UNICODE)
    static Db_perf01 Open (const wchar_t *name, const wchar_t *mode=L"s") throw (rdm_exception&);
#endif /*RDM_NO_UNICODE*/

    //! Db_perf01 Default Constructor
    Db_perf01 (void) throw ();
    /*! \brief Db_perf01 Constructor
     *
     * \param db A Db object used to create the Db_perf01
     */
    Db_perf01 (const Db& db) throw ();
    //! Db_perf01 Copy Constructor
    Db_perf01 (const Db_perf01&) throw ();

    //! Db_perf01 assignment operator
    Db_perf01& operator = (const Db&) throw (rdm_exception&);

    //! Db_perf01 assignment operator
    Db_perf01& operator = (const Db_perf01&) throw ();

    /*! \brief Db_perf01 destructor
     *
     * Remove a reference to the underlying database.  The database is closed when the last reference is gone.
     */
    ~Db_perf01 ();

    /*! \brief Create a new simple record
     *
     * \returns A singleton Cursor_simple object with default field values
     *
     * Create a new simple record using default values
     */
    Cursor_simple New_simple_record (void) const throw (rdm_exception&);

    /*! \brief Create a new simple record
     *
     * \param fields The field values for the new simple record
     * \returns A singleton Cursor_simple object with the specified field values
     *
     * Create a new simple record using the specified values
     */
    Cursor_simple New_simple_recordWithFieldValues (struct simple *fields) const throw (rdm_exception&);

    /*! \brief Obtain simple cursor position at the first simple record
     * \returns simple cursor positioned at the first simple record
     *
     * Create a new simple cursor for doing a table scan of the simple records.
     * The cursor is positioned at the first simple record
     */
    Cursor_simple Get_simple_records (void) const throw (rdm_exception&);

private:
    /*! \brief Db_perf01 reference operator */
    Db_perf01* operator & (void);
    /*! \brief Db_perf01 constructor */
    Db_perf01 (IDb *idb);
    /*! \brief Get the Db_perf01 dbType
     * \return The Db_perf01 dbType
     */
    static dbType Type (void);
};

/*! \brief The simple Cursor class
 *
 * This class provides the implementation for the simple Cursor
*/
class Cursor_simple : public Cursor {
public:
    /*! \brief Get the type for objects of this class
     * \return The type for Cursor_simple objects
     */
    static recordType Type (void);

    /*! Cursor_simple Default Constructor */
    Cursor_simple (void) throw ();
    /*! \brief Cursor_simple Constructor
     *
     * \param cursor A Cursor object used to create the Cursor_simple object
     */
    Cursor_simple (const Cursor& cursor) throw (rdm_exception&);
    /*! \brief Cursor_simple Copy Constructor */
    Cursor_simple (const Cursor_simple&) throw ();

    /*! \brief Cursor_simple assignment operator */
    Cursor_simple& operator = (const Cursor& cursor) throw (rdm_exception&);

    /*! \brief Cursor_simple assignment operator */
    Cursor_simple& operator = (const Cursor_simple& cursor) throw ();

    /*! \brief Set all of the field values of the record at the current cursor position
     * \param fields A const pointer with the field values to be written to the current cursor position
     */
    void SetFieldValues (const struct simple *fields) const throw (rdm_exception&);

    /*! \brief Get all of the field values for the record at the current cursor position
     * \param fields A pointer where the record values from the current cursor position will be written
     */
    void GetFieldValues (struct simple *fields) const throw (rdm_exception&);

    /*! \brief Set the int_col field value for the current record
     * \param int_col The field value that will be written to the record at the current cursor position
     */
    void Set_int_col (int32_t int_col_val) const throw (rdm_exception&);

    /*! \brief Get the int_col field value for the current record
     * \param int_col A pointer where the field value from the current cursor position will be written
     */
    void Get_int_col (int32_t *int_col_val) const throw (rdm_exception&);

    /*! \brief Set the sysnulls_ field value for the current record
     * \param sysnulls_ The field value that will be written to the record at the current cursor position
     */
    void Set_sysnulls_ (const char sysnulls__val[1][1]) const throw (rdm_exception&);

    /*! \brief Get the sysnulls_ field value for the current record
     * \param sysnulls_ A pointer where the field value from the current cursor position will be written
     */
    void Get_sysnulls_ (char sysnulls__val[1][1]) const throw (rdm_exception&);

    /*! \brief Cursor_simple Destructor
     *
     * Release the cursor
     */
    ~Cursor_simple ();

private:
    friend class Db_perf01;
};

}
#endif    /* PERF01_API_H */
